<?php

namespace Drupal\testsuite;

/**
 * Test Suite Services.
 */
class ShellExecService {

    /**
     * @var string
     */
    protected $_rootPath;

    protected $_version;


    /**
     * Constructs a new ShellExecService
     */
    public function __construct() {
        if($this->hasWebFolder()) {
            $this->_rootPath = dirname(dirname(dirname(dirname(dirname(__DIR__)))));
        } else {
            $this->_rootPath = dirname(dirname(dirname(dirname(__DIR__))));
        }
        // dd($this->_rootPath);
    }

    private function hasWebFolder() {
        $path = dirname(dirname(dirname(__DIR__)));
        if (preg_match('/\bweb\b/', $path)) return true;
        else return false;
    }

    /**
     * Gets the guzzle service.
     *
     * @return string
     *   The data pulled with shell_exec.
     */
    public function getData($statement) : string {
        // dd($_SERVER['DOCUMENT_ROOT']);
        $old_path = getcwd();
        chdir($this->_rootPath);
        $results = shell_exec($statement);
        chdir($old_path);
        return $results;
    }

    /**
     * Returns if the Test has errors.
     *
     * @return bool
     *   Returns if the test returns errors.
     */
    public function ifErrors($results) : bool {
        return preg_match('/ERRORS!/', $results);
    }

    /**
     * Returns if the class exist in the array.
     *
     * @return bool
     *   Returns if the class exist in the array.
     */
    public function classExists($class, $reportArray) : bool {
        foreach($reportArray as $module => $values) {
            foreach($values as $key => $value) {
                if($class == $value['class']) return true;
            }
        }

        return false;
    }

    /**
     * Returns the report heading.
     *
     * @return string
     *   Returns the result.
     */
    public function GetReportHeading($results) : string {
        $split_sections = preg_split('/\n\n\n/', $results);
        $second1_split_sections = preg_split('/\n\n/', $split_sections[0]);
        if(isset($split_sections[1])) {
            $second2_split_sections = preg_split('/\n\n/', $split_sections[1]);

            $section1_split = preg_split('/\n/', $second1_split_sections[1]);
            if($this->ifErrors($results)) {
                $mainArray = array(
                    $second1_split_sections[0],
                    $section1_split[0],
                    $second2_split_sections[0],
                    preg_split('/\n/', end($second2_split_sections))[1]
                );
            } else {
                $mainArray = array(
                    $second1_split_sections[0],
                    $section1_split[0],
                    $second2_split_sections[0],
                    $second2_split_sections[1]
                );
            }
        } else {
            $section1_split = preg_split('/\n/', $second1_split_sections[1]);
            $mainArray = array(
                $second1_split_sections[0],
                $section1_split[0]
            );
        }


        return implode("<br />", $mainArray);
    }

    /**
     * Returns a test result with errors.
     *
     * @return array
     *   Returns the result.
     */
    public function getReportMore($module, $results) : array {
        $classArray = array();
        $split_sections = preg_split('/\n\n\n/', $results);
        $second1_split_sections = preg_split('/\n\n/', $split_sections[0]);
        $section1_split = preg_split('/\n/', $second1_split_sections[1]);
        array_shift($section1_split);
        $classCount = 1;
        for ($i = 0; $i<count($section1_split); $i+=2) {
            if(substr($section1_split[$i], 0, 5) == 'Test ') {
            preg_match_all('/Drupal[^\s]+\s/', $section1_split[$i], $matches);
            $class = str_replace("'", "", $matches[0][0]);
            $class = str_replace(" ", "", $class);
            //dpm($class);
            if(!$this->classExists($class, $classArray)) {
                $classArray[$module][] = array(
                'class' => $class,
                'count' => $classCount
                );
                $classCountArray[$module][] = $classCount;
                $classCount = 0;
            }
            $classCount++;
            }
        }
        return $classArray;
    }

    /**
     * Returns a test result with errors.
     *
     * @return array
     *   Returns the result.
     */
    public function getErrorArray($module, $results) : array {
        $errorArray = array();
        $split_sections = preg_split('/\n\n\n/', $results);
        $second2_split_sections = preg_split('/\n\n/', $split_sections[1]);
        array_shift($second2_split_sections);
        array_shift($second2_split_sections);
        array_pop($second2_split_sections);
        for ($i = 0; $i<count($second2_split_sections); $i++) {
            $second2_sections = preg_split('/\n/', $second2_split_sections[$i]);
            $errorArray[$module][] = implode("<br />", $second2_sections);
        }
        return $errorArray;
    }
}